/**************************************************************************************
Author:			David Szybala
Program:		Fracture_Object.ms
Date:			2/10/10
Description:	Based on the script Fracture Voronoi (http://www.scriptspot.com/3ds-max/scripts/fracture-voronoi)
I made several enhancements to the tool.
***************************************************************************************/

try (destroyDialog fractureObjectRollout) catch()
	
(--put script into local context
	global fractureObjectRollout
	local fractureObjects = #()
	local fractureObjectIniFile = getDir #plugcfg + "\\FractureObject.ini"
	local fractureObjectRolloutPos

	/**************************************************************************************/
	fn Set_Fractures currentObj =
	(
		local totalFractures = fractureObjectRollout.fractureSpn.value
		local currentCopy = undefined
		local cutPlane = plane pos:selection[1].pos width:1 length:1 widthSegs:1 lengthSegs:1
		local bbox = nodeLocalBoundingBox currentObj
		local cutCoords = for i = 1 to totalFractures collect [(random bbox[1][1] bbox[2][1]), (random bbox[1][2] bbox[2][2]), (random bbox[1][3] bbox[2][3])]
		
		for i = 1 to totalFractures do
		(
			currentCopy = copy currentObj
			resetXForm currentCopy
			convertToMesh currentCopy
			setFaceSelection currentCopy #{}
			append fractureObjects currentCopy
			
			for x = 1 to totalFractures do
			(
				if x != i do
				(
					cutPlane.dir = cutCoords[x] - cutCoords[i]
					cutPlane.pos = ((cutCoords[i] + cutCoords[x]) / 2.0) - currentCopy.pos
					addModifier currentCopy (sliceModifier slice_type:2)
					currentCopy.slice.slice_plane.transform = cutPlane.transform
					addModifier currentCopy (cap_holes())
					convertToMesh currentCopy
				)--end if
			)--end for
		)--end for

		delete cutPlane
	)--end fn
	
	/**************************************************************************************/
	fn Set_ID =
	(
		for currentFracture in fractureObjects do
		(
			for currentFace in getFaceSelection currentFracture do setFaceMatID currentFracture currentFace fractureObjectRollout.matIDSpn.value
		)--end for
	)--end fn
	
	/**************************************************************************************/
	fn Process_Fractures currentObject =
	(
		local newMap = undefined
		
		if fractureObjectRollout.mapChanChk.checked do
		(
			for currentFracture in fractureObjects do
			(
				newMap = Uvwmap maptype:4 realWorldMapSize:true mapChannel:fractureObjectRollout.mapChanSpn.value
				addModifier currentFracture newMap
				--newMap.realWorldMapSize = false
				convertToMesh currentFracture
			)--end for
		)--end if
		
		if fractureObjectRollout.pivotsChk.checked do CenterPivot fractureObjects
		
		if fractureObjectRollout.wireRadio.state == 1 then
			for currentFracture in fractureObjects do currentFracture.wirecolor = color (random 0 255) (random 0 255) (random 0 255)
		else
			for currentFracture in fractureObjects do currentFracture.wirecolor = fractureObjectRollout.wireColorPckr.color
		
		if fractureObjectRollout.keepOriginalChk.checked and fractureObjectRollout.parentOriginalChk.checked do
			for currentFracture in fractureObjects do currentFracture.parent = currentObject
	)--end fn
	
	/**************************************************************************************/
	fn Set_Layer currentObject iterNum =
	(
		local newLayer = undefined
		local requestName = ""
		local counter = 1
		
		if fractureObjectRollout.keepIterChk.checked and fractureObjectRollout.iterSpn.value > 1 then
		(
			if iterNum < 10 then
				requestName = (currentObject.name + "_Fracture_Iter0" + (iterNum as string))
			else
				requestName = (currentObject.name + "_Fracture_Iter" + (iterNum as string))
		)--end if
		else
			requestName = (currentObject.name + "_Fracture")
		
		newLayer = LayerManager.getLayerFromName requestName
		
		if newLayer == undefined then
			newLayer = LayerManager.newLayerFromName requestName
		else
		(
			newLayer = LayerManager.getLayerFromName (requestName + "_" + (counter as string))
			
			while newLayer != undefined do
			(
				counter += 1
				newLayer = LayerManager.getLayerFromName (requestName + "_" + (counter as string))
			)--end while
		
			newLayer = LayerManager.newLayerFromName (requestName + "_" + (counter as string))
		)--end else
				
		newLayer.current = true
	)--end fn
	
	/**************************************************************************************/
	rollout fractureObjectRollout "Fracture Object" width:180 height:455
	(
		groupBox fractureGrp "Fractures" pos:[5,5] width:170 height:95
			label total1Lbl "Total (per iteration):" pos:[15,22] width:100 height:15
			spinner fractureSpn "" pos:[120,22] width:40 height:16 range:[2,99,5] type:#integer scale:1
			label total2Lbl "Total Iterations:" pos:[15,42] width:95 height:15
			spinner iterSpn "" pos:[120,42] width:40 height:16 range:[1,99,2] type:#integer scale:1
			label total3Lbl "Total Fractures (per object):" pos:[15,62] width:140 height:15
			label totalFracturesLabel "25" pos:[15,80] width:150 height:15
		
		groupBox newGrp "New Faces / Objects" pos:[5,105] width:170 height:100
			checkbox matIDChk "Material ID:" pos:[15,125] width:95 height:15 checked:true
			spinner matIDSpn "" pos:[120,124] width:40 height:16 range:[1,99,2] type:#integer scale:1
			checkbox mapChanChk "Map Channel:" pos:[15,145] width:95 height:15 checked:true
			spinner mapChanSpn "" pos:[120,144] width:40 height:16 range:[1,99,2] type:#integer scale:1
			label wirecolorLbl "Wirecolor:" pos:[15,165] width:50 height:15
			radioButtons wireRadio "" pos:[70,165] width:63 height:32 labels:#("Random", "Uniform") columns:1
			colorPicker wireColorPckr "" pos:[130,182] width:30 height:15 color:(color 0 200 100)
			
		groupBox optGrp "Options" pos:[5,210] width:170 height:125
			label seedLbl "Random Seed:" pos:[15,230] width:100 height:15
			spinner seedSpn "" pos:[120,230] width:40 height:16 range:[5,999,1] type:#integer scale:1
			checkbox keepIterChk "Keep Iterations" pos:[15,250] width:150 height:15
			checkbox keepOriginalChk "Keep Original Object" pos:[15,270] width:150 height:15 checked:true
				checkbox parentOriginalChk "Parent to Original Object" pos:[25,290] width:140 height:15
			checkbox pivotsChk "Center Pivots" pos:[15,310] width:150 height:15 checked:true
			
		button fractureBtn "Fracture Object(s)" pos:[5,340] width:170 height:20
		label progLbl "Standing by..." pos:[5,365] width:170 height:15
		progressBar progSubBar "" pos:[5,382] width:170 height:10
		progressBar progTotalBar "" pos:[5,395] width:170 height:10
		
		on fractureObjectRollout moved pos do setIniSetting fractureObjectIniFile "Dialog" "Position" (pos as string)
			
		-------------------------------------------------------------------
		on fractureSpn changed theValue do totalFracturesLabel.text = (((pow theValue iterSpn.value) as integer) as string)
		on iterSpn changed theValue do totalFracturesLabel.text = (((pow fractureSpn.value theValue) as integer) as string)
		on matIDChk changed theState do matIDSpn.enabled = theState
		on mapChanChk changed theState do mapChanSpn.enabled = theState
		on keepOriginalChk changed theState do parentOriginalChk.enabled = theState
		--------------------------------------------------------
		
		on fractureBtn pressed do
		(
			local currentObjects = selection as array
			local counter = 0
			local percent = 0
			
			progTotalBar.value = 0
			progSubBar.value = 0
			progTotalBar.color = [200,0,0]
			progSubBar.color = [200,0,0]
			fractureObjects = #()
			
			max create mode
			seed seedSpn.value
			
			with undo off
			(
				with redraw off
				(
					for i = 1 to currentObjects.count do
					(
						progLbl.text = ("Fracturing object " + (i as string) + " of " + (currentObjects.count as string) + "...")
						
						if canConvertTo currentObjects[i] Editable_mesh do
						(
							Set_Layer currentObjects[i] 1
							Set_Fractures currentObjects[i] --prime fracture
							if matIDChk.checked do Set_ID() --must be set before next cut
							if keepIterChk.checked or iterSpn == 1 do Process_Fractures currentObjects[i]
							
							for x = 2 to iterSpn.value do --iteration fractures
							(
								if keepIterChk.checked do Set_Layer currentObjects[i] x
								counter = fractureObjects.count
								
								for y = 1 to counter do Set_Fractures fractureObjects[y]
								
								if keepIterChk.checked then
								(
									for y = 1 to counter do deleteItem fractureObjects 1
									if matIDChk.checked do Set_ID() --must be set before next cut
									Process_Fractures currentObjects[i]
								)--end if
								else
								(
									for y = 1 to counter do
									(
										delete fractureObjects[1]
										deleteItem fractureObjects 1
									)--end for
									
									if matIDChk.checked do Set_ID() --must be set before next cut
								)--end else
								
								percent = 100.0 * x / iterSpn.value
								progSubBar.value = percent
								if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
							)--end for
							
							Process_Fractures currentObjects[i]
						)--end if
						
						if keepOriginalChk.checked == false do delete currentObjects[i]
						
						fractureObjects = #()
						
						percent = 100.0 * i / currentObjects.count
						progTotalBar.value = percent
						if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
					)--end for
				)--end redraw
				
				progTotalBar.color = [0,200,0]
				progSubBar.color = [0,200,0]
				progLbl.text = "Operation complete. Standing by..."
			)--end undo
		)--end on
	)--end rollout
	fractureObjectRolloutPos = execute (getIniSetting fractureObjectIniFile "Dialog" "Position")
	if fractureObjectRolloutPos == OK do fractureObjectRolloutPos = [100,100]
	if fractureObjectRolloutPos.x > sysinfo.desktopsize.x - 180 do fractureObjectRolloutPos.x = sysinfo.desktopsize.x - 185
	if fractureObjectRolloutPos.y > sysinfo.desktopsize.y - 470 do fractureObjectRolloutPos.y = sysinfo.desktopsize.y - 470
	if fractureObjectRolloutPos.x < 0 do fractureObjectRolloutPos.x = 0 
	if fractureObjectRolloutPos.y < 0 do fractureObjectRolloutPos.y = 0
	createDialog fractureObjectRollout 180 410 fractureObjectRolloutPos.x fractureObjectRolloutPos.y
)--end local context